/*  KASTL
 *  test_stable_partition.cpp
 *
 *  Author: Daouda Traore
 *
 *  September 2009
 */


#include <sys/times.h>
#include <sys/time.h>
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <kastl/algorithm>
#include "random.h"

typedef int type;

/** Return the number of seconds + micro seconds since the epoch
*/
inline double gettime()
{
  struct timeval tp;
  gettimeofday(&tp, 0);
  return (double)(tp.tv_sec) + (double)(tp.tv_usec)*1e-6;
}

template<class T>
bool IsOdd (T i)
{
 return ((int)i%2)==1;
}

template<class T>
bool sin_less_than_zero (T i)
{
 return sin(i) < 0;
}

template<class In, class Predicate>
void verif_partition(In first, In split, In last, Predicate pred)
{
   size_t s1 = split-first;
   size_t s2 = last-split;
   size_t i=0;
   size_t j=0;
   while((first!=split)&&(pred(*first++)))  i++;
   while((split!=last)&&(!pred(*split++)))  j++;
   std::cout << "s1= " << s1 << ", s2=" << s2 << std::endl;
   std::cout << "i= " << i << ", j=" << j << std::endl;
   if((i==s1)&&(j==s2)) std::cout << "The table has been well partionned..." << std::endl;
   else std::cout << "The table has not been well partionned..." << std::endl;

}


template<class In, class Out>
bool verif (In first, In last, Out res) {
     int i = 0;
     int cpt = 0;
     bool is_ok = true;
    if(last-first > 0) {
     if(((*first-*res) > 10e-7) || ((*res-*first) > 10e-7)) {
      std::cerr << "Erreur: res[" << i << "]=" << *res << "!=" << *first << std::endl;
      is_ok=false;
     }
     while(++first!=last) {
       ++res;
       i++;
       if(((*first-*res) > 10e-7) || ((*res-*first) > 10e-7)) {
        cpt++;
        if(cpt < 10)  std::cerr << "Erreur: res[" << i << "]=" << *res << "!=" << *first << std::endl;
        is_ok=false;
      }
     }
   }
 return is_ok;
}


struct doit {
   void operator()(int argc, char ** argv) {
  
  int cpu, l, n, iter;
  double t0;
  double avrg = 0;

  int v ;

  if (argc >1) n = atoi(argv[1]);
  else n = 10000;
  if (argc >2) cpu = atoi(argv[2]);
  else cpu = 1;
  if (argc >3) iter = atoi(argv[3]);
  else iter = 1;
   if (argc > 4) v = atoi(argv[4]);
  else v = 0;

  //kastl::stable_partition_macroloop_initsize = 30 ;
 
  t0 = gettime();
  type* input = new type[n];
  t0 = gettime() - t0;

  std::cout << "Time allocate:" << t0 << std::endl;
  Random random(42 + iter);

  type res;

  t0 = gettime();
  srand(0);
  for(int i = 0; i < n; ++i) {
    input[i] =  i;
  }
  t0 = gettime() - t0;
  std::cout << "Time init:" << t0 << std::endl;

  type *split;

  t0 = gettime();
  for (l=0; l<iter; ++l)
  {
       split = kastl::stable_partition(input, input+n, sin_less_than_zero<type>);
       /*kastl::PARTITION_BLOC_SELECTION = kastl::DYNAMIC;
       //kastl::PARTITION_BLOC_SELECTION = kastl::FIXE;
       split =  kastl::partition(input, input+n, sin_less_than_zero<type>);*/
  }
  t0 = (gettime() -t0);
  avrg += t0;
  avrg /= (double)iter;

  std::cout << "left partition size = " << split-input << std::endl;
  
   if(v) { //Verification only for stable_partition
       type* stl_res = new type[n];
       for(int i = 0; i < n; ++i) stl_res[i] =  i;
       std::stable_partition(stl_res, stl_res+n, sin_less_than_zero<type>);
       if(verif(input, input+n, stl_res))
       std::cout << "The table has been well partionned...." << std::endl;
       else std::cout << "The table hasn't been well partionned...." << std::endl;
       delete []  stl_res;
   }
   
   /*if(v) { //Verification only for partition
       verif_partition(input, split, input+n, sin_less_than_zero<type>);
   }*/



  std::cout << "Result-> cpu:" << cpu << "  :size: " << n << "  :time: " << avrg << std::endl;

  delete [] input;
 }
};

int main(int argc,char**argv){

  argc = 8;

  char** argvv = new char*[argc];
  argvv[0] = argv[0];
  argvv[1] = argv[1];
  argvv[2] = argv[2];
  argvv[3] = argv[3];
  argvv[4] = argv[4];
  argvv[5] = (char*) "--community";
  argvv[6] = (char*) "-thread.poolsize";
  argvv[7] = argv[2];
    
  a1::Community com = a1::System::join_community(argc,argvv);
  a1::ForkMain<doit>() (argc,argv);
  com.leave();
  a1::System::terminate();

}

