/*  STL
 *  test_accumulate.cpp
 *
 *  Author: Daouda Traore 
 *
 *  September 2009
 */

#include <sys/times.h>
#include <sys/time.h>
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <numeric>
#include "random.h"

typedef double type;

/** Return the number of seconds + micro seconds since the epoch
*/
inline double gettime()
{
  struct timeval tp;
  gettimeofday(&tp, 0);
  return (double)(tp.tv_sec) + (double)(tp.tv_usec)*1e-6;
}


/* The main thread
*/
int main(int argc, char** argv)
{
  int cpu, l, n, iter;
  double t0;
  double avrg = 0;


  if (argc >1) n = atoi(argv[1]);
  else n = 10000;
  if (argc >2) cpu = atoi(argv[2]);
  else cpu = 1;
  if (argc >3) iter = atoi(argv[3]);
  else iter = 1;

  t0 = gettime();
  type* input = new type[n];
  type* output = new type[n];
  t0 = gettime() - t0;

  std::cout << "Time allocate:" << t0 << std::endl;
  Random random(42 + iter);

  type res;

  t0 = gettime();
  for(int i = 0; i < n; ++i) {
    input[i] =  type(1)/type(i+1) ;
    if(i%2) input[i] = -input[i];
  }
  t0 = gettime() - t0;
  std::cout << "Time init:" << t0 << std::endl;

  t0 = gettime();
  for (l=0; l<iter; ++l)
  {
    res = std::accumulate(input, input+n, type(0));
  }
  t0 = (gettime() -t0);
  avrg += t0;
  avrg /= (double)iter;

   std::cout.precision(9);
   std::cout.setf(std::ios::fixed, std::ios_base::floatfield);
   // Verification of the result
   std::cout << "res = " << res << std::endl;

   std::cout << "Result-> cpu:" << cpu << "  :size: " << n << "  :time: " << avrg << std::endl;

  delete [] input;

  return 0;
}
