/*  STL
 *  test_find.cpp
 *
 *  Author: Daouda Traore
 *
 *  September 2009
 */


#include <sys/times.h>
#include <sys/time.h>
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <algorithm>
#include "random.h"

typedef int type;

/** Return the number of seconds + micro seconds since the epoch
*/
inline double gettime()
{
  struct timeval tp;
  gettimeofday(&tp, 0);
  return (double)(tp.tv_sec) + (double)(tp.tv_usec)*1e-6;
}

template <class T>
struct my_fct {
    T _val;
    my_fct(T val):_val(val) { };
    T operator()(T a){
         return  (_val < a);
    }
};

template<class T>
struct sin_equal_to_zero {
  T operator()(T val){
   return sin(val) == 0;
  }
};


/* The main thread
*/
int main(int argc, char** argv)
{
  int cpu, l, n, iter;
  double t0;
  double avrg = 0;
  type value;


  if (argc >1) n = atoi(argv[1]);
  else n = 10000;
  if (argc >2) cpu = atoi(argv[2]);
  else cpu = 1;
  if (argc >3) iter = atoi(argv[3]);
  else iter = 1;
  if (argc > 4) value = atoi(argv[4]);
  else value = 10000;

  t0 = gettime();
  type* input = new type[n];
  t0 = gettime() - t0;

  std::cout << "Time allocate:" << t0 << std::endl;
  Random random(42 + iter);

  type* pos_found;

  t0 = gettime();
  for(int i = 0; i < n; ++i) {
    input[i] = i+1 /*random.genrand_real3()*/;
  }

  t0 = gettime() - t0;
  std::cout << "Time init:" << t0 << std::endl;

  t0 = gettime();
  for (l=0; l<iter; ++l)
  {
    pos_found = std::find(input, input+n, value);
  }
  t0 = (gettime() -t0);
  avrg += t0;
  avrg /= (double)iter;

   // Verification of the result
   std::cout << "pos_found = " << pos_found-input << std::endl;

  std::cout << "Result-> cpu:" << cpu << "  :size: " << n << "  :time: " << avrg << std::endl;

  delete [] input;

  return 0;
}
