/*  Gnu C++ parallel mode
 *  test_set_intersection.cpp
 *
 *  Author: Daouda Traore
 *
 *  September 2009
 */


#include <sys/times.h>
#include <sys/time.h>
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <algorithm>
#include "random.h"

typedef double type;

/** Return the number of seconds + micro seconds since the epoch
*/
inline double gettime()
{
  struct timeval tp;
  gettimeofday(&tp, 0);
  return (double)(tp.tv_sec) + (double)(tp.tv_usec)*1e-6;
}


/* The main thread
*/
int main(int argc, char** argv)
{
  int cpu, l, n, iter;
  double t0;
  double avrg = 0;


  if (argc >1) n = atoi(argv[1]);
  else n = 10000;
  if (argc >2) cpu = atoi(argv[2]);
  else cpu = 1;
  if (argc >3) iter = atoi(argv[3]);
  else iter = 1;

  t0 = gettime();
  type* input = new type[n];
  type* input2 = new type[n];
  type* output = new type[2*n];
  t0 = gettime() - t0;

  std::cout << "Time allocate:" << t0 << std::endl;
  Random random(42 + iter);

  t0 = gettime();
  for(int i = 0; i < n; ++i) {
    input[i] = i+1; /*random.genrand_real3()*/; 
    input2[i] = 2*(i+1); /*random.genrand_real3()*/;
  }

  //std::sort(input, input+n);
  //std::sort(input2, input2+n);

  //int size_output = 2*n;
  //for(int i = 0; i < size_output; i++) output[i] = type(0);

  t0 = gettime() - t0;
  std::cout << "Time init:" << t0 << std::endl;

  type* res;

  t0 = gettime();
  for (l=0; l<iter; ++l)
  {
    res = std::set_intersection(input, input+n, input2, input2+n, output);
  }
  t0 = (gettime() -t0);
  avrg += t0;
  avrg /= (double)iter;

  std::cout << "size inserted elements =" << res-output << std::endl;
  std::cout << "Result-> cpu:" << cpu << "  :size: " << n << "  :time: " << avrg << std::endl;

  delete [] input;
  delete [] input2;
  delete [] output;
  
  return 0;
}
