/*  STL
 *  test_stable_partition.cpp
 *
 *  Author: Daouda Traore
 *
 *  September 2009
 */


#include <sys/times.h>
#include <sys/time.h>
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <algorithm>
#include "random.h"

typedef int type;

/** Return the number of seconds + micro seconds since the epoch
*/
inline double gettime()
{
  struct timeval tp;
  gettimeofday(&tp, 0);
  return (double)(tp.tv_sec) + (double)(tp.tv_usec)*1e-6;
}

template<class T>
bool IsOdd (T i)
{
 return ((int)i%2)==1;
}

template<class T>
bool sin_less_than_zero (T i)
{
 return sin(i) < 0;
}


int main(int argc,char**argv){
  
  int cpu, l, n, iter;
  double t0;
  double avrg = 0;

  if (argc >1) n = atoi(argv[1]);
  else n = 10000;
  if (argc >2) cpu = atoi(argv[2]);
  else cpu = 1;
  if (argc >3) iter = atoi(argv[3]);
  else iter = 1;
 
  t0 = gettime();
  type* input = new type[n];
  t0 = gettime() - t0;

  std::cout << "Time allocate:" << t0 << std::endl;
  Random random(42 + iter);

  type res;

  t0 = gettime();
  srand(0);
  for(int i = 0; i < n; ++i) {
    input[i] =  i;
  }
  t0 = gettime() - t0;
  std::cout << "Time init:" << t0 << std::endl;

  type *split;

  t0 = gettime();
  for (l=0; l<iter; ++l)
  {
       split = std::stable_partition(input, input+n, sin_less_than_zero<type>);
       //split = std::partition(input, input+n, sin_less_than_zero<type>);
  }
  t0 = (gettime() -t0);
  avrg += t0;
  avrg /= (double)iter;

  std::cout << "left partition size = " << split-input << std::endl;
  std::cout << "Result-> cpu:" << cpu << "  :size: " << n << "  :time: " << avrg << std::endl;

  delete [] input;
 }

